@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 1996-2021 The Octave Project Developers
+// Copyright (C) 1996-2022 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -36,17 +36,12 @@
 
 #if defined (OCTAVE_USE_WINDOWS_API)
 #  include <windows.h>
-#  include <shlwapi.h>
-#endif
-
-#if (defined (OCTAVE_HAVE_WINDOWS_FILESYSTEM) && ! defined (OCTAVE_HAVE_POSIX_FILESYSTEM))
-#  include <algorithm>
-#  include "localcharset-wrapper.h"
-#  include "uniconv-wrappers.h"
+#  include "unwind-prot.h"
+#else
+#  include "canonicalize-file-name-wrapper.h"
 #endif
 
 #include "areadlink-wrapper.h"
-#include "canonicalize-file-name-wrapper.h"
 #include "dir-ops.h"
 #include "file-ops.h"
 #include "file-stat.h"
@@ -54,7 +49,7 @@
 #include "lo-sysdep.h"
 #include "oct-env.h"
 #include "oct-locbuf.h"
-#include "oct-passwd.h"
+#include "oct-password.h"
 #include "quit.h"
 #include "stat-wrappers.h"
 #include "str-vec.h"
@@ -75,12 +70,12 @@ namespace octave
   // any word separation get desired behavior.
   static const char *default_suffixes[] = { " ", "\n", ":", nullptr };
 
-  static size_t
-  tilde_find_prefix (const std::string& s, size_t& len)
+  static std::size_t
+  tilde_find_prefix (const std::string& s, std::size_t& len)
   {
     len = 0;
 
-    size_t s_len = s.length ();
+    std::size_t s_len = s.length ();
 
     if (s_len == 0 || s[0] == '~')
       return 0;
@@ -89,11 +84,11 @@ namespace octave
 
     if (! prefixes.empty ())
       {
-        for (size_t i = 0; i < s_len; i++)
+        for (std::size_t i = 0; i < s_len; i++)
           {
             for (int j = 0; j < prefixes.numel (); j++)
               {
-                size_t pfx_len = prefixes[j].length ();
+                std::size_t pfx_len = prefixes[j].length ();
 
                 if (prefixes[j] == s.substr (i, pfx_len))
                   {
@@ -110,14 +105,14 @@ namespace octave
   // Find the end of a tilde expansion in S, and return the index
   // of the character which ends the tilde definition.
 
-  static size_t
+  static std::size_t
   tilde_find_suffix (const std::string& s)
   {
-    size_t s_len = s.length ();
+    std::size_t s_len = s.length ();
 
     string_vector suffixes = sys::file_ops::tilde_additional_suffixes;
 
-    size_t i = 0;
+    std::size_t i = 0;
 
     for ( ; i < s_len; i++)
       {
@@ -128,7 +123,7 @@ namespace octave
           {
             for (int j = 0; j < suffixes.numel (); j++)
               {
-                size_t sfx_len = suffixes[j].length ();
+                std::size_t sfx_len = suffixes[j].length ();
 
                 if (suffixes[j] == s.substr (i, sfx_len))
                   return i;
@@ -144,9 +139,9 @@ namespace octave
   static std::string
   isolate_tilde_prefix (const std::string& fname)
   {
-    size_t f_len = fname.length ();
+    std::size_t f_len = fname.length ();
 
-    size_t len = 1;
+    std::size_t len = 1;
 
     while (len < f_len && ! sys::file_ops::is_dir_sep (fname[len]))
       len++;
@@ -160,7 +155,7 @@ namespace octave
   static std::string
   tilde_expand_word (const std::string& filename)
   {
-    size_t f_len = filename.length ();
+    std::size_t f_len = filename.length ();
 
     if (f_len == 0 || filename[0] != '~')
       return std::string (filename);
@@ -174,7 +169,7 @@ namespace octave
 
     std::string username = isolate_tilde_prefix (filename);
 
-    size_t user_len = username.length ();
+    std::size_t user_len = username.length ();
 
     std::string dirname;
 
@@ -291,22 +286,22 @@ namespace octave
           {
             std::string result;
 
-            size_t name_len = name.length ();
+            std::size_t name_len = name.length ();
 
             // Scan through S expanding tildes as we come to them.
 
-            size_t pos = 0;
+            std::size_t pos = 0;
 
             while (1)
               {
                 if (pos > name_len)
                   break;
 
-                size_t len;
+                std::size_t len;
 
                 // Make START point to the tilde which starts the expansion.
 
-                size_t start = tilde_find_prefix (name.substr (pos), len);
+                std::size_t start = tilde_find_prefix (name.substr (pos), len);
 
                 result.append (name.substr (pos, start));
 
@@ -317,7 +312,7 @@ namespace octave
                 // Make FINI be the index of one after the last character of the
                 // username.
 
-                size_t fini = tilde_find_suffix (name.substr (pos));
+                std::size_t fini = tilde_find_suffix (name.substr (pos));
 
                 // If both START and FINI are zero, we are all done.
 
@@ -362,14 +357,14 @@ namespace octave
 
       std::string dirname (const std::string& path)
       {
-        size_t ipos = path.find_last_of (dir_sep_chars ());
+        std::size_t ipos = path.find_last_of (dir_sep_chars ());
 
         return (ipos != std::string::npos) ? path.substr (0, ipos) : "";
       }
 
       std::string tail (const std::string& path)
       {
-        size_t ipos = path.find_last_of (dir_sep_chars ());
+        std::size_t ipos = path.find_last_of (dir_sep_chars ());
 
         if (ipos != std::string::npos)
           ipos++;
@@ -387,8 +382,8 @@ namespace octave
           retval = path;
         else
           {
-            size_t n = path.length ();
-            for (size_t i = 0; i < n; i++)
+            std::size_t n = path.length ();
+            for (std::size_t i = 0; i < n; i++)
               {
                 if (path[i] == '/')
                   retval += dir_sep_char();
@@ -702,44 +697,45 @@ namespace octave
 
       std::string retval;
 
-#if (defined (OCTAVE_HAVE_WINDOWS_FILESYSTEM) && ! defined (OCTAVE_HAVE_POSIX_FILESYSTEM))
-      // On Windows, convert to locale charset before passing to
-      // canonicalize_file_name, and convert back to UTF-8 after that.
+      // FIXME:  Consider replacing this with std::filesystem::canonical on all
+      // platforms once we allow using C++17.
 
-      // FIXME: This only allows non-ASCII characters in the file or path that
-      // can be encoded in the locale charset.
-      // Consider replacing this with std::filesystem::canonical once we allow
-      // using C++17.
+#if defined (OCTAVE_USE_WINDOWS_API)
+      // open file handle
+      std::wstring wname = u8_to_wstring (name);
+      HANDLE h_file = CreateFileW (wname.c_str (), GENERIC_READ,
+                                   FILE_SHARE_READ, nullptr, OPEN_EXISTING,
+                                   FILE_FLAG_BACKUP_SEMANTICS, nullptr);
 
-      const char *locale = octave_locale_charset_wrapper ();
-      const uint8_t *name_u8 = reinterpret_cast<const uint8_t *>
-                                 (name.c_str ());
-      size_t length = 0;
-      char *name_locale = octave_u8_conv_to_encoding (locale, name_u8,
-                                                      name.length () + 1,
-                                                      &length);
-
-      if (name_locale)
+      if (h_file == INVALID_HANDLE_VALUE)
         {
-          char *tmp_locale =
-            octave_canonicalize_file_name_wrapper (name_locale);
-          free (name_locale);
-
-          if (tmp_locale)
-            {
-              char *tmp = reinterpret_cast<char *>
-                            (octave_u8_conv_from_encoding (locale, tmp_locale,
-                                                           strlen (tmp_locale),
-                                                           &length));
-              free (tmp_locale);
-
-              if (tmp)
-                {
-                  retval = std::string (tmp, length);
-                  free (tmp);
-                }
-            }
+          msg = "Unable to open file \"" + name + "\"";
+          return retval;
         }
+
+      unwind_action close_file_handle (CloseHandle, h_file);
+
+      const std::size_t buf_size = 32767;
+      wchar_t buffer[buf_size] = L"";
+
+      // query canonical name
+      DWORD len = GetFinalPathNameByHandleW (h_file, buffer, buf_size,
+                                             FILE_NAME_NORMALIZED);
+      if (len >= buf_size)
+        {
+          msg = "Error querying normalized name for \"" + name + "\"";
+          return retval;
+        }
+
+      retval = u8_from_wstring (std::wstring (buffer, len));
+
+      // remove prefix
+      // "Normal" paths are prefixed by "\\?\".
+      // UNC paths are prefixed by "\\?\UNC\".
+      if (retval.compare (0, 8, R"(\\?\UNC\)") == 0)
+        retval = retval.erase (2, 6);
+      else if (retval.compare (0, 4, R"(\\?\)") == 0)
+        retval = retval.erase (0, 4);
 #else
       char *tmp = octave_canonicalize_file_name_wrapper (name.c_str ());
 
@@ -748,63 +744,10 @@ namespace octave
           retval = tmp;
           free (tmp);
         }
-#endif
-
-#if (defined (OCTAVE_HAVE_WINDOWS_FILESYSTEM) && ! defined (OCTAVE_HAVE_POSIX_FILESYSTEM))
-      // Canonical Windows file separator is backslash.
-      std::replace (retval.begin (), retval.end (), '/', '\\');
-#endif
-
-#if defined (OCTAVE_USE_WINDOWS_API)
-      std::wstring w_tmp;
-      bool strip_marker = true;
-      if (retval.empty ())
-        {
-          // For UNC paths, take the input as is.
-          // Also translate forward slashes to backslashes.
-          std::string name_backsl = name;
-          std::replace (name_backsl.begin (), name_backsl.end (), '/', '\\');
-          if (name_backsl.compare (0, 2, "\\\\") == 0)
-            {
-              w_tmp = u8_to_wstring (name_backsl);
-              strip_marker = false;
-              wchar_t canon_path[MAX_PATH];
-              if (PathCanonicalizeW (canon_path, w_tmp.c_str ()))
-                w_tmp = std::wstring (canon_path);
-            }
-        }
-      else
-        w_tmp = L"\\\\?\\" + u8_to_wstring (retval);
-
-      if (! w_tmp.empty ())
-        {
-          // Get a more canonical name wrt case and full names
-          // FIXME: To make this work on partitions that don't store short file
-          // names, use FindFirstFileW on each component of the path.
-          // Insufficient access permissions on parent folders might make this
-          // tricky.
-
-          // Parts of the path that wouldn't fit into a short 8.3 file name are
-          // copied as is by GetLongPathNameW.  To also get the correct case
-          // for these parts, first convert to short file names and than back
-          // to long.
-          wchar_t buffer[32767] = L"";
-          int w_len = GetShortPathNameW (w_tmp.c_str (), buffer, 32767);
-          w_len = GetLongPathNameW (buffer, buffer, 32767);
-
-          if (! strip_marker)
-            retval = u8_from_wstring (std::wstring (buffer, w_len));
-          else if (w_len > 4)
-            retval = u8_from_wstring (std::wstring (buffer+4, w_len-4));
-
-          // If root is a drive, use an upper case letter for the drive letter.
-          if (retval.length () > 1 && retval[1] == ':')
-            retval[0] = toupper (retval[0]);
-        }
-#endif
 
       if (retval.empty ())
         msg = std::strerror (errno);
+#endif
 
       return retval;
     }
