@@ -1,4 +1,9 @@
-## Copyright (C) 1995-2019 Kurt Hornik
+########################################################################
+##
+## Copyright (C) 1995-2022 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {} moment (@var{x}, @var{p})
@@ -121,12 +128,9 @@
 ## Can easily be made to work for continuous distributions (using quad)
 ## as well, but how does the general case work?
 
-## Author: KH <Kurt.Hornik@wu-wien.ac.at>
-## Description: Compute moments
-
 function m = moment (x, p, opt1, opt2)
 
-  if (nargin < 2 || nargin > 4)
+  if (nargin < 2)
     print_usage ();
   endif
 
@@ -202,9 +206,8 @@ endfunction
 %!assert (moment (1, 2, 4), 0)
 
 ## Test input validation
-%!error moment ()
-%!error moment (1)
-%!error moment (1, 2, 3, 4, 5)
+%!error <Invalid call> moment ()
+%!error <Invalid call> moment (1)
 %!error <X must be a non-empty numeric matrix> moment (['A'; 'B'], 2)
 %!error <X must be a non-empty numeric matrix> moment (ones (2,0,3), 2)
 %!error <P must be a numeric scalar> moment (1, true)
